"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const platform_1 = require("@rxstack/platform");
const exceptions_1 = require("@rxstack/exceptions");
const _ = require("lodash");
const utils_1 = require("./utils");
exports.validateUnique = (options) => {
    return (event) => __awaiter(this, void 0, void 0, function* () {
        utils_1.restrictToOperations(event.eventType, [platform_1.OperationEventsEnum.PRE_EXECUTE]);
        const service = event.injector.get(options.service);
        const input = event.request.body;
        const data = _.isArray(event.getData()) ? event.getData() : [event.getData()];
        yield processUniqueValidation(service, input, data, options);
    });
};
const processUniqueValidation = (service, input, data, options) => __awaiter(this, void 0, void 0, function* () {
    const filteredData = data.filter((v) => _.isObject(v));
    if (_.isArray(input)) {
        for (let i = 0; i < input.length; i++) {
            yield validateUniqueItem(service, input[i], filteredData, options);
        }
    }
    else {
        yield validateUniqueItem(service, input, filteredData, options);
    }
});
const validateUniqueItem = (service, input, data, options) => __awaiter(this, void 0, void 0, function* () {
    const criteria = buildCriteria(input, options.properties);
    const method = options.method || 'findMany';
    const result = yield service[method].call(service, { where: criteria, skip: 0, limit: 1 });
    if (shouldThrowException(result, data, options)) {
        throwException(input, options);
    }
});
const buildCriteria = (input, properties) => {
    const criteria = {};
    properties.forEach(field => {
        criteria[field] = { '$eq': utils_1.getProperty(input, field) };
    });
    return criteria;
};
const shouldThrowException = (result, data, options) => {
    return false === (result.length === 0 || exists(result[0], data, options.properties));
};
const exists = (result, data, properties) => {
    const partialResult = _.pick(result, properties);
    return !!_.find(data, (value) => _.isEqual(partialResult, _.pick(value, properties)));
};
const throwException = (input, options) => {
    const message = options.message || 'Value is not unique';
    const exception = new exceptions_1.BadRequestException('Validation Failed');
    exception.data = {
        errors: [
            {
                path: options.errorPath,
                value: _.get(input, options.properties[0]),
                message: message
            }
        ]
    };
    throw exception;
};
//# sourceMappingURL=validate-unique.js.map